@@ -89,8 +89,8 @@ try:
     from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT \
         as NavigationToolbar
 
-except ImportError:
-    raise RuntimeError("Unable to find a usable Qt version.")
+except ImportError as e:
+    raise RuntimeError(f"Unable to find a usable Qt version {e}.")
 
 
 # The file selector dialog on OSX is buggy, so switching allowed file extensions
@@ -142,9 +142,9 @@ def run_gui(settings, test_mode=False):
     # Start up the Qt application and exit when it does
     app = QApplication(sys.argv[:1])
     mainwindow = MainWindow(settings)
-    mainwindow.show()
     if test_mode:
-        mainwindow.defer_close()
+        mainwindow.defer_close(app)
+    mainwindow.show()
     return app.exec_()
 
 
@@ -590,9 +590,9 @@ class MainWindow(QMainWindow):
             self.load_files(self.defer_load)
             self.defer_load = None
 
-    def defer_close(self):
+    def defer_close(self, qapp):
         self.close_timer = QTimer(self)
-        self.close_timer.timeout.connect(self.close)
+        self.close_timer.timeout.connect(qapp.quit, Qt.QueuedConnection)
         self.close_timer.start()
 
     def shorten_titles(self, titles):
@@ -1031,7 +1031,7 @@ class NewTestDialog(QDialog):
         if (p, s) == (0, 0):
             if not self.aborted:
                 elapsed = time.time() - self.start_time
-                self.progressBar.setValue(100 * elapsed / self.total_time)
+                self.progressBar.setValue(int(100 * elapsed / self.total_time))
         else:
             fn = os.path.join(self.settings.DATA_DIR,
                               self.settings.DATA_FILENAME)
@@ -1053,17 +1053,24 @@ class QPlainTextLogger(loggers.Handler):
         self.widget = widget or QPlainTextEdit(parent)
         self.widget.setFont(font)
         self.widget.setReadOnly(True)
+        self.widget.destroyed.connect(self.destroy_widget)
 
         self.statusbar = statusbar
         self.timeout = timeout
 
     def emit(self, record):
+        if self.widget is None:
+            return
+
         msg = self.format(record)
         self.widget.appendPlainText(msg)
 
         if self.statusbar:
             self.statusbar.showMessage(record.message, self.timeout)
 
+    def destroy_widget(self):
+        self.widget = None
+
     def write(self, p):
         pass
 
